/*
 * Copyright (C) 2016 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef UTILS_PROFILER_FILE_H
#define UTILS_PROFILER_FILE_H

#include <sys/types.h>
#include <string>

namespace profiler {

// Class to manipulate filesystem.
class ProfilerFile {
 public:
  explicit ProfilerFile(std::string path);

  bool Exists() const;

  size_t GetSize() const;

  bool Delete() const;

  const std::string& GetPath() const { return path_; }

  const std::string GetFileName() const;

 private:
  std::string path_;
};
}  // namespace profiler

#endif  // UTILS_PROFILER_FILE_H
